@@ -5,7 +5,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# Author: Kyle Lahnakoski (kyle@lahnakoski.com)
+# Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
 from __future__ import absolute_import, division, unicode_literals
 
@@ -25,12 +25,12 @@ from mo_threads import Queue, THREAD_STOP, Thread, Till
 from mo_times import Duration, MINUTE
 from mo_times.dates import datetime2unix
 from pyLibrary.convert import bytes2base64
-from pyLibrary.env.rollover_index import RolloverIndex
+from jx_elasticsearch.rollover_index import RolloverIndex
 
 MAX_BAD_COUNT = 5
 LOG_STRING_LENGTH = 2000
-PAUSE_AFTER_GOOD_INSERT = 1
-PAUSE_AFTER_BAD_INSERT = 60
+PAUSE_AFTER_GOOD_INSERT = 60
+PAUSE_AFTER_BAD_INSERT = 600
 
 
 class StructuredLogger_usingElasticSearch(StructuredLogger):
